@@ -1,5 +1,7 @@
 #!/usr/bin/python
 # has to be a full import due to Ansible 2.0 compatibility
+from ipaddress import AddressValueError, IPv4Interface, IPv6Interface, NetmaskValueError
+
 import six
 from ansible.module_utils.basic import AnsibleModule
 
@@ -35,26 +37,14 @@ options:
     description:
       - Fully qualified URL to the Waldur.
     required: true
-  cidr:
-    description:
-      - A CIDR the security group rule is applied to.
-       It is required if 'rules' are not provided.
   description:
     description:
       - A description of the security group.
     required: false
-  from_port:
-    description:
-      - The lowest port value the security group rule is applied to.
-        It is required if 'rules' are not provided.
   interval:
     default: 20
     description:
       - An interval of the security group state polling.
-  protocol:
-    description:
-      - A protocol the security group rule is applied to.
-        It is required if 'rules' are not provided.
   name:
     description:
       - The name of the security group.
@@ -66,8 +56,10 @@ options:
   rules:
     description:
       - A list of security group rules to be applied to the security group.
-        A rule consists of 4 fields: 'to_port', 'from_port', 'cidr' and 'protocol'
-        It is required if 'to_port', 'from_port', 'cidr' and 'protocol' are not specified.
+        A rule consists of 5 fields: 'to_port', 'from_port', 'protocol', 'ethertype',
+        'direction' and either 'cidr' or 'remote_group' (remote group name).
+        'ethertype' (IPv4 by default) and 'direction' (ingress by default)
+        parameters are optional.
   state:
     choices:
       - present
@@ -75,10 +67,6 @@ options:
     default: present
     description:
       - Should the resource be present or absent.
-  to_port:
-    description:
-      - The highest port value the security group rule is applied to.
-        It is required if 'rules' are not provided.
   tags:
     description:
       - List of tags that will be added to the security group on provisioning.
@@ -138,10 +126,11 @@ EXAMPLES = '''
         api_url: https://waldur.example.com:8000/api
         tenant: VPC #1
         description: http only
-        from_port: 80
-        to_port: 80
-        cidr: 0.0.0.0/0
-        protocol: tcp
+        rules:
+          - from_port: 80
+            to_port: 80
+            cidr: 0.0.0.0/0
+            protocol: tcp
         state: present
         name: classic-web
         tags:
@@ -184,6 +173,99 @@ EXAMPLES = '''
         description: empty group
         state: present
         name: empty
+
+- name: add security group using remote group
+  hosts: localhost
+  tasks:
+    - name: create security group with a link to remote group
+      waldur_os_security_group:
+        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
+        api_url: https://waldur.example.com:8000/api
+        tenant: VPC
+        description: group depending on a remote group
+        rules:
+        - from_port: 80
+          to_port: 80
+          remote_group: web
+          protocol: tcp
+        state: present
+        name: group_remote_group
+
+- name: add security group with cidr ethertype
+  hosts: localhost
+  gather_facts: no
+  tasks:
+    - name: create security group with cidr ethertype
+      waldur_os_security_group:
+        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
+        api_url: https://waldur.example.com:8000/api
+        tenant: waldur-dev-infra
+        description: some descr
+        rules:
+        - from_port: 80
+          to_port: 80
+          cidr: 0.0.0.0/00
+          protocol: tcp
+          ethertype: IPv4
+        state: present
+        name: sec-group-with-ethertype
+
+- name: add security group with IPv6 cidr
+  hosts: localhost
+  gather_facts: no
+  tasks:
+    - name: create security group with cidr ethertype
+      waldur_os_security_group:
+        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
+        api_url: https://waldur.example.com:8000/api
+        tenant: waldur-dev-infra
+        description: some descr
+        rules:
+        - from_port: 80
+          to_port: 80
+          cidr: 2002::/16
+          protocol: tcp
+          ethertype: IPv6
+        state: present
+        name: sec-group-with-ethertype
+
+- name: add security group with ingress direction
+  hosts: localhost
+  gather_facts: no
+  tasks:
+    - name: create security group with direction
+      waldur_os_security_group:
+        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
+        api_url: https://waldur.example.com:8000/api
+        tenant: waldur-dev-infra
+        description: some descr
+        rules:
+        - from_port: 80
+          to_port: 80
+          cidr: 0.0.0.0/00
+          protocol: tcp
+          direction: ingress
+        state: present
+        name: sec-group-with-direction
+
+- name: add security group with egress direction
+  hosts: localhost
+  gather_facts: no
+  tasks:
+    - name: create security group with direction
+      waldur_os_security_group:
+        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
+        api_url: https://waldur.example.com:8000/api
+        tenant: waldur-dev-infra
+        description: some descr
+        rules:
+        - from_port: 80
+          to_port: 80
+          cidr: 0.0.0.0/00
+          protocol: tcp
+          direction: egress
+        state: present
+        name: sec-group-with-direction
 '''
 
 
@@ -193,14 +275,46 @@ def send_request_to_waldur(client, module):
     tenant = module.params['tenant']
     name = module.params['name']
     description = module.params.get('description') or ''
-    rules = module.params.get('rules') or [
-        {
-            'from_port': module.params['from_port'],
-            'to_port': module.params['to_port'],
-            'cidr': module.params['cidr'],
-            'protocol': module.params['protocol'],
-        }
-    ] if module.params['cidr'] else []
+    rules = module.params['rules']
+    for rule in rules:
+        for item in ['from_port', 'to_port', 'protocol']:
+            if item not in rule:
+                module.fail_json(msg='A rule must contain %s parameter.' % item)
+
+        if 'cidr' in rule and 'remote_group' in rule:
+            module.fail_json(
+                msg='Either cidr or remote_group must be specified, not both.')
+
+        if 'remote_group' in rule:
+            remote_group = client.get_security_group(tenant, rule['remote_group'])
+            rule['remote_group'] = remote_group['url']
+        elif 'cidr' in rule:
+            address = rule['cidr']
+            if 'ethertype' not in rule or rule['ethertype'] == 'IPv4':
+                try:
+                    IPv4Interface(address)
+                except (AddressValueError, NetmaskValueError) as e:
+                    module.fail_json(msg='Invalid IPv4 address %s: %s' % (address, e))
+                else:
+                    rule['ethertype'] = 'IPv4'
+            elif rule['ethertype'] == 'IPv6':
+                try:
+                    IPv6Interface(address)
+                except (AddressValueError, NetmaskValueError) as e:
+                    module.fail_json(msg='Invalid IPv6 address %s: %s' % (address, e))
+            else:
+                module.fail_json(msg='Invalid ethertype: %s' % rule['ethertype'])
+        else:
+            module.fail_json(msg='Either cidr or remote_group must be specified.')
+
+        if 'direction' not in rule:
+            rule['direction'] = 'ingress'
+        else:
+            if rule['direction'] not in ['ingress', 'egress']:
+                module.fail_json(
+                    msg='Invalid direction %s expected ingress or egress' %
+                        rule['direction']
+                )
 
     security_group = client.get_security_group(tenant, name)
     present = module.params['state'] == 'present'
@@ -236,27 +350,12 @@ def send_request_to_waldur(client, module):
 
 def main():
     fields = waldur_resource_argument_spec(
-        rules=dict(type='list', required=False),
-        from_port=dict(type='str'),
-        to_port=dict(type='str'),
-        cidr=dict(type='str'),
-        protocol=dict(type='str', choices=['tcp', 'udp', 'icmp']),
+        rules=dict(type='list', required=False, default=[]),
         project=dict(type='str', required=False),
         tenant=dict(type='str', required=True),
     )
-    required_together = [['from_port', 'to_port', 'cidr', 'protocol']]
-    mutually_exclusive = [
-        ['from_port', 'rules'],
-        ['to_port', 'rules'],
-        ['cidr', 'rules'],
-        ['protocol', 'rules'],
-    ]
-    required_one_of = []
     module = AnsibleModule(
         argument_spec=fields,
-        required_together=required_together,
-        required_one_of=required_one_of,
-        mutually_exclusive=mutually_exclusive,
     )
 
     client = waldur_client_from_module(module)
